/* A replacement for CfgSort */

/* Load the rexxUtil functions -------------------------------------------- */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

Prog="Logical Config.Sys Sort, by Doug Bissett"
Progver="003"

Out_File_Order='first',
               'NVC',
               'IFS',
               'USERIFS',
               'PATHS',
               'JAVA',
               'System',
               'APM',
               'COM',
               'USB',
               'PCCard',
               'Network',
               'Misc',
               'XFree86',
               'MMOS2',
               'VIDEO',
               'RSJ',
               'Lotus',
               'WATCOM',
               'VACPP',
               'Opti',
               'FireWire',
               'USER',
               'UNKNOWN',
               'last'

PARSE ARG INPUT_FILE OUTPUT_FILE '/'PARM1 '/'PARM2

INPUT_FILE=STRIP(INPUT_FILE)
OUTPUT_FILE=STRIP(OUTPUT_FILE)
PARM1=Translate(STRIP(PARM1))
PARM2=Translate(STRIP(PARM2))

SAY 'INPUT_FILE='INPUT_FILE
SAY 'OUTPUT_FILE='OUTPUT_FILE
/* SAY 'PARMS='PARM1' 'PARM2 */

If PARM1='E'|PARM2='E' then Emphasize=1 
else Emphasize=0 
If PARM1='B'|PARM2='B' then KeepBlanks=1 
else KeepBlanks=0 

ORIGINAL_INPUT_FILE=INPUT_FILE
ORIGINAL_OUTPUT_FILE=OUTPUT_FILE

/* sanity checks */
if INPUT_FILE=''|OUTPUT_FILE='' then do
  SAY
  SAY "Incorrect command"
  SAY
  SAY " the correct command is:"
  SAY " LCSS.CMD input_file output_file [/E] [/B]"
  SAY
  SAY "/E means to emphasize the section headers"
  SAY "/B means to keep an existing blank line following a line (only one)"
  SAY
  SAY "Press any key to exit"
  answer=sysGetKey()
  EXIT 1
end

/* Get a file name to use as a backup file */
do i=1 to 999
  OUTPUT_BACKUP=ORIGINAL_OUTPUT_FILE'.'i
  rc=sysFileTree(OUTPUT_BACKUP,'stem','FO')
  if rc=2 then do
    SAY 
    SAY "Out of memory error detected.Program will terminate."
    SAY "Tap a key to exit. "
    answer=sysGetKey()
    EXIT 2
  end  
  if stem.0=0 then leave
end

/* Check to see if the output file already exists */
rc=sysFileTree(OUTPUT_FILE,'stem','FO')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end
  
if stem.0>0 then do
  SAY
  SAY 'WARNING: Continuing will replace the original 'OUTPUT_FILE
  SAY
  SAY 'A backup of 'OUTPUT_FILE' will be created as 'OUTPUT_BACKUP
  SAY
  SAY 'Type the word YES, in upper case, then enter to continue'
  SAY 
  PARSE PULL answer
  if answer = 'YES' then do
    '@copy 'OUTPUT_FILE OUTPUT_BACKUP
    w=lastPos("\",OUTPUT_BACKUP)
    OUTPUTBACKUP=right(OUTPUT_BACKUP,w)
    rc=SysPutEA(OUTPUT_BACKUP, '.LONGNAME', OUTPUTBACKUP)
    rc=sysFileDelete(OUTPUT_FILE)
    if rc>0 then do
      SAY
      SAY "Error deleting "OUTPUT_FILE
      SAY "Program will terminate"
      SAY "Tap a key to exit"
      answer=sysGetKey()
      EXIT 2
    end
    If INPUT_FILE=OUTPUT_FILE then do
      /* need to use the backup file as input */
      INPUT_FILE=OUTPUT_BACKUP
    end
  end
  else do
    EXIT 1
  end
end

/* Get rid of old files */
rc=sysFileTree('*.sorted','stem','FO')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end  
if stem.0>0 then do
  do x=1 to stem.0
    rc=sysFileDelete(stem.x)
    if rc>0 then do
      SAY
      SAY "Error deleting "stem.x
      SAY "Program will terminate"
      SAY "Tap a key to exit"
      answer=sysGetKey()
      EXIT 2
    end
  end
end

infile.0=0
x=0
/* Get input file into memory, removing lines with 'REM ['  */
Do while lines(INPUT_FILE)
  in_line=lineIn(INPUT_FILE)
  if left(in_line,5)\='REM [' then do
    if in_line=''&KeepBlanks then do
      in_line='KEEP BLANK'
    end
    x=x+1
    infile.x=in_line
  end
end
infile.0=x
rc=stream(INPUT_FILE,'c','close')

/* Open template file(s), and go through the input file, sorting lines into sections */
/* Do the user file first, if it exists */
UBVer="Not used"
rc=sysFileTree('LCSS.usr','stem','F')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end
if stem.0>0 then do
  /* First, get the userbase file version */
  testline=lineIn(LCSS.usr)
  If left(testline,18)="; userbase version" then do
    UBVer=right(testline,length(testline)-2)
  end
  else
  do
    UBVer="old"
  end
  /* now, go through the userbase file, selecting matching lines from input file */
  Do while lines(LCSS.usr)
    testline = LineIn(LCSS.usr)
    if (left(testline,1)\=';')&(testline \='') then do
      If left(testline,2)='**' then do
        PARSE VAR testline '** ' GroupName JUNK
      end
      else do
        testfor = translate(testline)
        Do x=1 to infile.0
          testline=translate(infile.x)
          if pos(testfor,testline)>0 then do
            /* keep previous REM with the line */
            y=x-1
            if translate(left(infile.y,3))='REM' then do
              rc=lineout(GroupName'.sorted',infile.y)
              infile.y=''
            end
            rc=Lineout(GroupName'.sorted',infile.x)
            /* keep following blank line with the line, if keep blank lines requested */
            If KeepBlanks then do
              y=x+1
              if infile.y='KEEP BLANK' then do
                rc=lineout(GroupName'.sorted','')
                infile.y=''
              end
            end
            infile.x=''
          end
        end
      end 
    end
  end
end

/* Now do the main file */
/* First, get the database file version */
testline=lineIn(LCSS.dat)
If left(testline,18)="; database version" then do
  DBVer=right(testline,length(testline)-2)
end
else
do
  DBVer="old"
end
/* now, go through the database file, selecting matching lines from input file */
Do while lines(LCSS.dat)
  testline = LineIn(LCSS.dat)
  if (left(testline,1)\=';')&(testline \='') then do
    If left(testline,2)='**' then do
      PARSE VAR testline '** ' GroupName JUNK
    end
    else do
      testfor = translate(testline)
      Do x=1 to infile.0
        testline=translate(infile.x)
        if pos(testfor,testline)>0 then do
          /* keep previous REM with the line */
          y=x-1
          if translate(left(infile.y,3))='REM' then do
            rc=lineout(GroupName'.sorted',infile.y)
            infile.y=''
          end
          rc=Lineout(GroupName'.sorted',infile.x)
          /* keep following blank line with the line, if keep blank lines requested */
          If KeepBlanks then do
            y=x+1
            if infile.y='KEEP BLANK' then do
              rc=lineout(GroupName'.sorted','')
              infile.y=''
            end
          end
          infile.x=''
        end
      end
    end 
  end
end

/* Now handle anything that is left (unknown) */
Do x=1 to infile.0
  if infile.x\=''&infile.x\='KEEP BLANK' then do
    rc=Lineout('unknown.sorted',infile.x)
  end
end

/* close the output files */
rc=sysFileTree('*.sorted','stem','FO')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end  
if stem.0>0 then do
  do x=1 to stem.0
    rc=stream(stem.x,'c','close')
  end
end

/* now, delete any pre-existing output file */
rc=sysFileTree(OUTPUT_FILE,'stem','FO')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end  
if stem.0>0 then do
  do x=1 to stem.0
    rc=sysFileDelete(stem.x)
    if rc>0 then do
      SAY
      SAY "Error deleting "OUTPUT_FILE
      SAY "Program will terminate"
      SAY "Tap a key to exit"
      answer=sysGetKey()
      EXIT 2
    end
  end
end

/* Add some words to the new file */
rc=lineOut(OUTPUT_FILE,'REM ['Prog)
rc=lineOut(OUTPUT_FILE,'REM [Program version: 'ProgVer)
rc=lineOut(OUTPUT_FILE,'REM [Database version: 'DBVer)
rc=lineOut(OUTPUT_FILE,'REM [Userbase version: 'UBVer)
rc=lineOut(OUTPUT_FILE,'REM [Config.Sys Created on 'Date()' at 'Time())

/* Now assemble the new CONFIG.SYS */

Do until Out_File_Order=''
  PARSE UPPER VAR Out_File_Order Fname Out_File_Order
  rc=sysFileTree(Fname'.sorted','stem','F')
  if rc=2 then do
    SAY 
    SAY "Out of memory error detected.Program will terminate."
    SAY "Tap a key to exit. "
    answer=sysGetKey()
    EXIT 2
  end  
  if stem.0>0 then do
    rc=lineOut(OUTPUT_FILE,'')
    if Emphasize=1 then Do
      rc=lineOut(OUTPUT_FILE,'REM [################')
    end
    rc=lineOut(OUTPUT_FILE,'REM ['Fname' section')
    if Emphasize=1 then Do
      rc=lineOut(OUTPUT_FILE,'REM [################')
    end
    if Fname="UNKNOWN" then do
      SAY 'Unknown lines found in 'ORIGINAL_INPUT_FILE
      SAY 'Please inspect the output.'
      rc=lineOut(OUTPUT_FILE,'REM [Unknown statements')
      rc=lineOut(OUTPUT_FILE,'REM [These statements may belong in other sections')
      rc=lineOut(OUTPUT_FILE,'REM [They may be deleted, if they are really not relevant')
      rc=lineOut(OUTPUT_FILE,'REM [Please READ the LCSS.README, to determine what')
      rc=lineOut(OUTPUT_FILE,'REM [to do about these lines.')
    end
    do while lines(Fname'.sorted')
      inline=lineIn(Fname'.sorted')
      rc=lineOut(OUTPUT_FILE,inline)
    end
    rc=stream(Fname'.sorted','c','close')
    rc=sysFileDelete(Fname'.sorted')
    if rc>0 then do
      SAY
      SAY "Error deleting "Fname".sorted"
      SAY "Program will terminate"
      SAY "Tap a key to exit"
      answer=sysGetKey()
      EXIT 2
    end
  end
end
 
/* Now, check for left over *.sorted files (should be none) */
rc=sysFileTree('*.sorted','stem','FO')
if rc=2 then do
  SAY 
  SAY "Out of memory error detected.Program will terminate."
  SAY "Tap a key to exit. "
  answer=sysGetKey()
  EXIT 2
end  
if stem.0>0 then do
  SAY
  SAY "WARNING: Something in the database file is not correct"
  SAY "         Please inspect the LCSS.usr file, for unknown groups"
  SAY "         Or report the problem, if it is in the LCSS.dat file"
  SAY 
  SAY "The bad group names are: "
  Do i=1 to stem.0
    w=lastpos('\',stem.i)
    w=length(stem.i) - w
    groupname=right(stem.i,w)
    w=lastpos('.',groupname)
    w=w-1
    SAY left(groupname,w)
  end
  SAY
  SAY "Tap a key to finish. The bad entry will be added to the end of the output file"
  answer=sysGetKey()

  rc=lineOut(OUTPUT_FILE,'REM [WARNING: Something in the database file is not correct')
  rc=lineOut(OUTPUT_FILE,'REM [Please inspect the LCSS.usr file, for unknown groups')
  rc=lineOut(OUTPUT_FILE,'REM [Or report the problem, if it is in the LCSS.dat file')
  rc=lineOut(OUTPUT_FILE,'REM [The following lines are included in an unknown group:')

  Do i=1 to stem.0
    FName= stem.i  
    rc=lineOut(OUTPUT_FILE,'REM [')
    do while lines(Fname)
      inline=lineIn(Fname)
      rc=lineOut(OUTPUT_FILE,inline)
    end
    rc=stream(Fname,'c','close')
    rc=sysFileDelete(Fname)
    if rc>0 then do
      SAY
      SAY "Error deleting "Fname
      SAY "Program will terminate"
      SAY "Tap a key to exit"
      answer=sysGetKey()
      EXIT 2
    end
  end
end

/* close the output file */
rc=stream(OUTPUT_FILE,'c','close')

/* now, make sure that the last line is blank */
Do while lines(OUTPUT_FILE)
  in_line=lineIn(OUTPUT_FILE)
end
rc=stream(OUTPUT_FILE,'c','close')
If in_line \= '' then do
  rc=lineOut(OUTPUT_FILE,'')
end 

EXIT 0